#!/usr/bin/env python3

# Copyright (C) 2021 VMware Inc.

# Simple module to convert cloud-init network config V1 to V2
# This satisfies most of our use cases at the moment
# Need to add support to handle all other configs as needed


import yaml
import ipaddress


def parse_subnet_routes(cfg, conf):
    # place holder for routes inside subnets
    pass


def parse_subnets(v2cfg, cfg, conf):
    ip4prefixlen = None
    for i in conf:
        for k, v in i.items():
            if k == 'type':
                if v == 'dhcp' or v == 'dhcp4':
                    cfg['id0']['dhcp4'] = True
                if v == 'dhcp6':
                    cfg['id0']['dhcp6'] = True
                if v == 'static' or v == 'static6':
                    cfg['id0']['dhcp4'] = False
                    cfg['id0']['dhcp6'] = False
            elif k == 'address':
                cfg['id0']['addresses'] = [v]
            elif k == 'netmask':
                ip4 = ipaddress.IPv4Network((0, v))
                # TODO: need to make use of this
                ip4prefixlen = ip4.prefixlen
            elif k == 'gateway':
                cfg['id0']['gateway4'] = v
            elif k == 'dns_nameservers':
                try:
                    cfg['id0']['nameservers'].update({'addresses' : v})
                except KeyError:
                    cfg['id0']['nameservers'] = {'addresses' : v}
            elif k == 'dns_search':
                try:
                    cfg['id0']['nameservers'].update({'search' : v})
                except KeyError:
                    cfg['id0']['nameservers'] = {'search' : v}
            elif k == 'routes':
                parse_subnet_routes(cfg, v)

    v2cfg['ethernets'].update(cfg)


def parse_phy(v2cfg, conf):
    cfg = dict(id0 = dict())
    for k, v in conf.items():
        if k == 'name':
            cfg['id0']['set-name'] = v
        elif k == 'mac_address':
            cfg['id0']['match'] = {'macaddress' : v}
        elif k == 'subnets':
            parse_subnets(v2cfg, cfg, v)
        elif k == 'mtu':
            cfg['id0']['mtu'] = v

    v2cfg['ethernets'].update(cfg)


def parse_routes(v2cfg, conf):
    cfg = {}
    for k, v in conf.items():
        if k == 'destination':
            cfg['to'] = v
        elif k == 'gateway':
            cfg['via'] = v
        elif k == 'metric':
            cfg[k] = v

    try:
        v2cfg['ethernets']['id0']['routes'].append(cfg)
    except KeyError:
        v2cfg['ethernets']['id0']['routes'] = [cfg]


def parse_nameserver(v2cfg, conf):
    cfg = {}
    for k, v in conf.items():
        if k == 'search':
            cfg[k] = v
        elif k == 'address':
            cfg['addresses'] = v

    try:
        v2cfg['ethernets']['id0']['nameservers'].update(cfg)
    except KeyError:
        v2cfg['ethernets']['id0']['nameservers'] = cfg


def v1tov2(cfg):
    v2cfg = dict(
        ethernets =  dict()
    )

    for i in cfg:
        for k, v in i.items():
            if k != 'type':
                continue
            if v == 'physical':
                parse_phy(v2cfg, i)
            elif v == 'route':
                parse_routes(v2cfg, i)
            elif v == 'nameserver':
                parse_nameserver(v2cfg, i)

    return v2cfg
